@@ -1,5 +1,4 @@
 ;;;; Emacs Lisp help for writing curl code. ;;;;
-;;;; $Id: curl-style.el,v 1.11 2004/05/24 12:12:09 bagder Exp $
 
 ;;; The curl hacker's C conventions.
 ;;; See the sample.emacs file on how this file can be made to take
@@ -20,6 +19,13 @@
     )
   "Curl C Programming Style")
 
+(defun curl-code-cleanup ()
+  "no docs"
+  (interactive)
+  (untabify (point-min) (point-max))
+  (delete-trailing-whitespace)
+)
+
 ;; Customizations for all of c-mode, c++-mode, and objc-mode
 (defun curl-c-mode-common-hook ()
   "Curl C mode hook"
@@ -28,12 +34,12 @@
   (setq tab-width 8
 	indent-tabs-mode nil		; Use spaces. Not tabs.
 	comment-column 40
-	c-font-lock-extra-types (append '("bool" "CURL" "CURLcode" "ssize_t" "size_t" "socklen_t" "fd_set" "time_t" "curl_off_t" "curl_socket_t" "in_addr_t" "CURLSHcode" "CURLMcode"))
+	c-font-lock-extra-types (append '("bool" "CURL" "CURLcode" "ssize_t" "size_t" "curl_socklen_t" "fd_set" "time_t" "curl_off_t" "curl_socket_t" "in_addr_t" "CURLSHcode" "CURLMcode" "Curl_addrinfo"))
 	)
   ;; keybindings for C, C++, and Objective-C.  We can put these in
   ;; c-mode-base-map because of inheritance ...
   (define-key c-mode-base-map "\M-q" 'c-fill-paragraph)
-  (define-key c-mode-base-map "\M-m" 'delete-trailing-whitespace)
+  (define-key c-mode-base-map "\M-m" 'curl-code-cleanup)
   (setq c-recognize-knr-p nil)
   ;;; (add-hook 'write-file-hooks 'delete-trailing-whitespace t)
   (setq show-trailing-whitespace t)
