@@ -20,13 +20,17 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: polarssl.h,v 1.10 2009-02-12 20:48:43 danf Exp $
  ***************************************************************************/
+#include "setup.h"
 
 #ifdef USE_POLARSSL
 
 CURLcode Curl_polarssl_connect(struct connectdata *conn, int sockindex);
 
+CURLcode Curl_polarssl_connect_nonblocking(struct connectdata *conn,
+                                           int sockindex,
+                                           bool *done);
+
 /* tell PolarSSL to close down all open information regarding connections (and
    thus session ID caching etc) */
 void Curl_polarssl_close_all(struct SessionHandle *data);
@@ -40,8 +44,9 @@ int Curl_polarssl_shutdown(struct connectdata *conn, int sockindex);
 
 /* API setup for PolarSSL */
 #define curlssl_init() (1)
-#define curlssl_cleanup()
+#define curlssl_cleanup() Curl_nop_stmt
 #define curlssl_connect Curl_polarssl_connect
+#define curlssl_connect_nonblocking Curl_polarssl_connect_nonblocking
 #define curlssl_session_free(x)  Curl_polarssl_session_free(x)
 #define curlssl_close_all Curl_polarssl_close_all
 #define curlssl_close Curl_polarssl_close
