@@ -34,52 +34,47 @@ typedef enum {
   SMTP_EHLO,
   SMTP_HELO,
   SMTP_STARTTLS,
-  SMTP_UPGRADETLS, /* asynchronously upgrade the connection to SSL/TLS
-                      (multi mode only) */
-  SMTP_AUTHPLAIN,
-  SMTP_AUTHLOGIN,
-  SMTP_AUTHPASSWD,
-  SMTP_AUTHCRAMMD5,
-  SMTP_AUTHNTLM,
-  SMTP_AUTHNTLM_TYPE2MSG,
+  SMTP_UPGRADETLS,  /* asynchronously upgrade the connection to SSL/TLS
+                       (multi mode only) */
+  SMTP_AUTH_PLAIN,
+  SMTP_AUTH_LOGIN,
+  SMTP_AUTH_PASSWD,
+  SMTP_AUTH_CRAMMD5,
+  SMTP_AUTH_DIGESTMD5,
+  SMTP_AUTH_DIGESTMD5_RESP,
+  SMTP_AUTH_NTLM,
+  SMTP_AUTH_NTLM_TYPE2MSG,
   SMTP_AUTH,
-  SMTP_MAIL, /* MAIL FROM */
-  SMTP_RCPT, /* RCPT TO */
+  SMTP_MAIL,        /* MAIL FROM */
+  SMTP_RCPT,        /* RCPT TO */
   SMTP_DATA,
   SMTP_POSTDATA,
   SMTP_QUIT,
-  SMTP_LAST  /* never used */
+  SMTP_LAST         /* never used */
 } smtpstate;
 
 /* smtp_conn is used for struct connection-oriented data in the connectdata
    struct */
 struct smtp_conn {
   struct pingpong pp;
-  char *domain;    /* what to send in the EHLO */
-  size_t eob;         /* number of bytes of the EOB (End Of Body) that has been
-                         received thus far */
-  unsigned int authmechs;       /* Accepted authentication methods. */
-  unsigned int authused;  /* Authentication method used for the connection */
-  smtpstate state; /* always use smtp.c:state() to change state! */
-  struct curl_slist *rcpt;
-  bool ssldone; /* is connect() over SSL done? only relevant in multi mode */
+  char *domain;            /* Client address/name to send in the EHLO */
+  size_t eob;              /* Number of bytes of the EOB (End Of Body) that
+                              have been received so far */
+  unsigned int authmechs;  /* Accepted authentication mechanisms */
+  unsigned int authused;   /* Auth mechanism used for the connection */
+  smtpstate state;         /* Always use smtp.c:state() to change state! */
+  struct curl_slist *rcpt; /* Recipient list */
+  bool ssldone;            /* Is connect() over SSL done? only relevant in
+                              multi mode */
 };
 
-/* Authentication mechanism flags. */
-#define SMTP_AUTH_LOGIN         0x0001
-#define SMTP_AUTH_PLAIN         0x0002
-#define SMTP_AUTH_CRAM_MD5      0x0004
-#define SMTP_AUTH_DIGEST_MD5    0x0008
-#define SMTP_AUTH_GSSAPI        0x0010
-#define SMTP_AUTH_EXTERNAL      0x0020
-#define SMTP_AUTH_NTLM          0x0040
-
 extern const struct Curl_handler Curl_handler_smtp;
 extern const struct Curl_handler Curl_handler_smtps;
 
 /* this is the 5-bytes End-Of-Body marker for SMTP */
 #define SMTP_EOB "\x0d\x0a\x2e\x0d\x0a"
 #define SMTP_EOB_LEN 5
+#define SMTP_EOB_FIND_LEN 3
 
 /* if found in data, replace it with this string instead */
 #define SMTP_EOB_REPL "\x0d\x0a\x2e\x2e"
