@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2012, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -76,7 +76,6 @@ my $ipvnum = 4;     # server IPv number (4 or 6)
 my $proto = 'ftp';  # default server protocol
 my $srcdir;         # directory where ftpserver.pl is located
 my $srvrname;       # server name for presentation purposes
-my $grok_eprt;
 
 my $path   = '.';
 my $logdir = $path .'/log';
@@ -117,6 +116,8 @@ local *SFWRITE;   # used to write to primary connection
 local *DREAD;     # used to read from secondary connection
 local *DWRITE;    # used to write to secondary connection
 
+my $sockfilt_timeout = 5;  # default timeout for sockfilter eXsysreads
+
 #**********************************************************************
 # global vars which depend on server protocol selection
 #
@@ -127,23 +128,41 @@ my @welcome;      # text returned to client upon connection
 #**********************************************************************
 # global vars customized for each test from the server commands file
 #
-my $ctrldelay;    # set if server should throttle ctrl stream
-my $datadelay;    # set if server should throttle data stream
-my $retrweirdo;   # set if ftp server should use RETRWEIRDO
-my $retrnosize;   # set if ftp server should use RETRNOSIZE
-my $pasvbadip;    # set if ftp server should use PASVBADIP
-my $nosave;       # set if ftp server should not save uploaded data
-my %customreply;  #
-my %customcount;  #
-my %delayreply;   #
+my $ctrldelay;     # set if server should throttle ctrl stream
+my $datadelay;     # set if server should throttle data stream
+my $retrweirdo;    # set if ftp server should use RETRWEIRDO
+my $retrnosize;    # set if ftp server should use RETRNOSIZE
+my $pasvbadip;     # set if ftp server should use PASVBADIP
+my $nosave;        # set if ftp server should not save uploaded data
+my $nodataconn;    # set if ftp srvr doesn't establish or accepts data channel
+my $nodataconn425; # set if ftp srvr doesn't establish data ch and replies 425
+my $nodataconn421; # set if ftp srvr doesn't establish data ch and replies 421
+my $nodataconn150; # set if ftp srvr doesn't establish data ch and replies 150
+my $support_capa;  # set if server supports capability command
+my $support_auth;  # set if server supports authentication command
+my %customreply;   #
+my %customcount;   #
+my %delayreply;    #
 
 #**********************************************************************
 # global variables for to test ftp wildcardmatching or other test that
 # need flexible LIST responses.. and corresponding files.
 # $ftptargetdir is keeping the fake "name" of LIST directory.
+#
 my $ftplistparserstate;
 my $ftptargetdir;
 
+#**********************************************************************
+# global variables used when running a ftp server to keep state info
+# relative to the secondary or data sockfilt process. Values of these
+# variables should only be modified using datasockf_state() sub, given
+# that they are closely related and relationship is a bit awkward.
+#
+my $datasockf_state = 'STOPPED'; # see datasockf_state() sub
+my $datasockf_mode = 'none';     # ['none','active','passive']
+my $datasockf_runs = 'no';       # ['no','yes']
+my $datasockf_conn = 'no';       # ['no','yes']
+
 #**********************************************************************
 # global vars used for signal handling
 #
@@ -205,6 +224,141 @@ sub ftpmsg {
   # better on windows/cygwin
 }
 
+#**********************************************************************
+# eXsysread is a wrapper around perl's sysread() function. This will
+# repeat the call to sysread() until it has actually read the complete
+# number of requested bytes or an unrecoverable condition occurs.
+# On success returns a positive value, the number of bytes requested.
+# On failure or timeout returns zero.
+#
+sub eXsysread {
+    my $FH      = shift;
+    my $scalar  = shift;
+    my $nbytes  = shift;
+    my $timeout = shift; # A zero timeout disables eXsysread() time limit
+    #
+    my $time_limited = 0;
+    my $timeout_rest = 0;
+    my $start_time = 0;
+    my $nread  = 0;
+    my $rc;
+
+    $$scalar = "";
+
+    if((not defined $nbytes) || ($nbytes < 1)) {
+        logmsg "Error: eXsysread() failure: " .
+               "length argument must be positive\n";
+        return 0;
+    }
+    if((not defined $timeout) || ($timeout < 0)) {
+        logmsg "Error: eXsysread() failure: " .
+               "timeout argument must be zero or positive\n";
+        return 0;
+    }
+    if($timeout > 0) {
+        # caller sets eXsysread() time limit
+        $time_limited = 1;
+        $timeout_rest = $timeout;
+        $start_time = int(time());
+    }
+
+    while($nread < $nbytes) {
+        if($time_limited) {
+            eval {
+                local $SIG{ALRM} = sub { die "alarm\n"; };
+                alarm $timeout_rest;
+                $rc = sysread($FH, $$scalar, $nbytes - $nread, $nread);
+                alarm 0;
+            };
+            $timeout_rest = $timeout - (int(time()) - $start_time);
+            if($timeout_rest < 1) {
+                logmsg "Error: eXsysread() failure: timed out\n";
+                return 0;
+            }
+        }
+        else {
+            $rc = sysread($FH, $$scalar, $nbytes - $nread, $nread);
+        }
+        if($got_exit_signal) {
+            logmsg "Error: eXsysread() failure: signalled to die\n";
+            return 0;
+        }
+        if(not defined $rc) {
+            if($!{EINTR}) {
+                logmsg "Warning: retrying sysread() interrupted system call\n";
+                next;
+            }
+            if($!{EAGAIN}) {
+                logmsg "Warning: retrying sysread() due to EAGAIN\n";
+                next;
+            }
+            if($!{EWOULDBLOCK}) {
+                logmsg "Warning: retrying sysread() due to EWOULDBLOCK\n";
+                next;
+            }
+            logmsg "Error: sysread() failure: $!\n";
+            return 0;
+        }
+        if($rc < 0) {
+            logmsg "Error: sysread() failure: returned negative value $rc\n";
+            return 0;
+        }
+        if($rc == 0) {
+            logmsg "Error: sysread() failure: read zero bytes\n";
+            return 0;
+        }
+        $nread += $rc;
+    }
+    return $nread;
+}
+
+#**********************************************************************
+# read_mainsockf attempts to read the given amount of output from the
+# sockfilter which is in use for the main or primary connection. This
+# reads untranslated sockfilt lingo which may hold data read from the
+# main or primary socket. On success returns 1, otherwise zero.
+#
+sub read_mainsockf {
+    my $scalar  = shift;
+    my $nbytes  = shift;
+    my $timeout = shift; # Optional argument, if zero blocks indefinitively
+    my $FH = \*SFREAD;
+
+    if(not defined $timeout) {
+        $timeout = $sockfilt_timeout + ($nbytes >> 12);
+    }
+    if(eXsysread($FH, $scalar, $nbytes, $timeout) != $nbytes) {
+        my ($fcaller, $lcaller) = (caller)[1,2];
+        logmsg "Error: read_mainsockf() failure at $fcaller " .
+               "line $lcaller. Due to eXsysread() failure\n";
+        return 0;
+    }
+    return 1;
+}
+
+#**********************************************************************
+# read_datasockf attempts to read the given amount of output from the
+# sockfilter which is in use for the data or secondary connection. This
+# reads untranslated sockfilt lingo which may hold data read from the
+# data or secondary socket. On success returns 1, otherwise zero.
+#
+sub read_datasockf {
+    my $scalar = shift;
+    my $nbytes = shift;
+    my $timeout = shift; # Optional argument, if zero blocks indefinitively
+    my $FH = \*DREAD;
+
+    if(not defined $timeout) {
+        $timeout = $sockfilt_timeout + ($nbytes >> 12);
+    }
+    if(eXsysread($FH, $scalar, $nbytes, $timeout) != $nbytes) {
+        my ($fcaller, $lcaller) = (caller)[1,2];
+        logmsg "Error: read_datasockf() failure at $fcaller " .
+               "line $lcaller. Due to eXsysread() failure\n";
+        return 0;
+    }
+    return 1;
+}
 
 sub sysread_or_die {
     my $FH     = shift;
@@ -312,15 +466,27 @@ sub sendcontrol {
     my $log;
     foreach $log (@_) {
         my $l = $log;
-        $l =~ s/[\r\n]//g;
+        $l =~ s/\r/[CR]/g;
+        $l =~ s/\n/[LF]/g;
         logmsg "> \"$l\"\n";
     }
 }
 
-# Send data to the client on the data stream
-
+#**********************************************************************
+# Send data to the FTP client on the data stream when data connection
+# is actually established. Given that this sub should only be called
+# when a data connection is supposed to be established, calling this
+# without a data connection is an indication of weak logic somewhere.
+#
 sub senddata {
     my $l;
+    if($datasockf_conn eq 'no') {
+        logmsg "WARNING: Detected data sending attempt without DATA channel\n";
+        foreach $l (@_) {
+            logmsg "WARNING: Data swallowed: $l\n"
+        }
+        return;
+    }
     foreach $l (@_) {
       if(!$datadelay) {
         # spit it all out at once
@@ -391,6 +557,8 @@ sub protocolsetup {
     }
     elsif($proto eq 'pop3') {
         %commandfunc = (
+            'CAPA' => \&CAPA_pop3,
+            'AUTH' => \&AUTH_pop3,
             'RETR' => \&RETR_pop3,
             'LIST' => \&LIST_pop3,
         );
@@ -453,25 +621,40 @@ sub close_dataconn {
 
     my $datapid = processexists($datasockf_pidfile);
 
+    logmsg "=====> Closing $datasockf_mode DATA connection...\n";
+
     if(!$closed) {
-        logmsg "* disconnect data connection\n";
         if($datapid > 0) {
+            logmsg "Server disconnects $datasockf_mode DATA connection\n";
             print DWRITE "DISC\n";
             my $i;
             sysread DREAD, $i, 5;
         }
+        else {
+            logmsg "Server finds $datasockf_mode DATA connection already ".
+                   "disconnected\n";
+        }
     }
     else {
-        logmsg "data connection already disconnected\n";
+        logmsg "Server knows $datasockf_mode DATA connection is already ".
+               "disconnected\n";
     }
-    logmsg "=====> Closed data connection\n";
 
-    logmsg "* quit sockfilt for data (pid $datapid)\n";
     if($datapid > 0) {
         print DWRITE "QUIT\n";
         waitpid($datapid, 0);
         unlink($datasockf_pidfile) if(-f $datasockf_pidfile);
+        logmsg "DATA sockfilt for $datasockf_mode data channel quits ".
+               "(pid $datapid)\n";
     }
+    else {
+        logmsg "DATA sockfilt for $datasockf_mode data channel already ".
+               "dead\n";
+    }
+
+    logmsg "=====> Closed $datasockf_mode DATA connection\n";
+
+    datasockf_state('STOPPED');
 }
 
 ################
@@ -524,7 +707,7 @@ sub DATA_smtp {
                 $size = hex($1);
             }
 
-            sysread \*SFREAD, $line, $size;
+            read_mainsockf(\$line, $size);
 
             $ulsize += $size;
             print FILE $line if(!$nosave);
@@ -642,6 +825,50 @@ sub FETCH_imap {
 ################ POP3 commands
 ################
 
+sub CAPA_pop3 {
+    my ($testno) = @_;
+    my @data = ();
+
+    if(!$support_capa) {
+        push @data, "-ERR Unsupported command: 'CAPA'\r\n";
+    }
+    else {
+        push @data, "+OK List of capabilities follows\r\n";
+        push @data, "USER\r\n";
+        if($support_auth) {
+            push @data, "SASL UNKNOWN\r\n";
+        }
+        push @data, "IMPLEMENTATION POP3 pingpong test server\r\n";
+        push @data, ".\r\n";
+    }
+
+    for my $d (@data) {
+        sendcontrol $d;
+    }
+
+    return 0;
+}
+
+sub AUTH_pop3 {
+    my ($testno) = @_;
+    my @data = ();
+
+    if(!$support_auth) {
+        push @data, "-ERR Unsupported command: 'AUTH'\r\n";
+    }
+    else {
+        push @data, "+OK List of supported mechanisms follows\r\n";
+        push @data, "UNKNOWN\r\n";
+        push @data, ".\r\n";
+    }
+
+    for my $d (@data) {
+        sendcontrol $d;
+    }
+
+    return 0;
+}
+
 sub RETR_pop3 {
      my ($testno) = @_;
      my @data;
@@ -678,8 +905,9 @@ sub RETR_pop3 {
          sendcontrol $d;
      }
 
-     # end with the magic 5-byte end of mail marker
-     sendcontrol "\r\n.\r\n";
+     # end with the magic 3-byte end of mail marker, assumes that the
+     # mail body ends with a CRLF!
+     sendcontrol ".\r\n";
 
      return 0;
 }
@@ -701,8 +929,8 @@ my @pop3list=(
          sendcontrol $d;
      }
 
-     # end with the magic 5-byte end of listing marker
-     sendcontrol "\r\n.\r\n";
+     # end with the magic 3-byte end of listing marker
+     sendcontrol ".\r\n";
 
      return 0;
 }
@@ -790,6 +1018,25 @@ my @ftpdir=("total 20\r\n",
 "drwxrwxrwx   2 98       1            512 Oct 30 14:33 pub\r\n",
 "dr-xr-xr-x   5 0        1            512 Oct  1  1997 usr\r\n");
 
+    if($datasockf_conn eq 'no') {
+        if($nodataconn425) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "425 Can't open data connection\r\n";
+        }
+        elsif($nodataconn421) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "421 Connection timed out\r\n";
+        }
+        elsif($nodataconn150) {
+            sendcontrol "150 Opening data connection\r\n";
+            # client shall timeout
+        }
+        else {
+            # client shall timeout
+        }
+        return 0;
+    }
+
     if($ftplistparserstate) {
       @ftpdir = ftp_contentlist($ftptargetdir);
     }
@@ -805,6 +1052,26 @@ my @ftpdir=("total 20\r\n",
 
 sub NLST_ftp {
     my @ftpdir=("file", "with space", "fake", "..", " ..", "funny", "README");
+
+    if($datasockf_conn eq 'no') {
+        if($nodataconn425) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "425 Can't open data connection\r\n";
+        }
+        elsif($nodataconn421) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "421 Connection timed out\r\n";
+        }
+        elsif($nodataconn150) {
+            sendcontrol "150 Opening data connection\r\n";
+            # client shall timeout
+        }
+        else {
+            # client shall timeout
+        }
+        return 0;
+    }
+
     logmsg "pass NLST data on data connection\n";
     for(@ftpdir) {
         senddata "$_\r\n";
@@ -908,6 +1175,25 @@ sub SIZE_ftp {
 sub RETR_ftp {
     my ($testno) = @_;
 
+    if($datasockf_conn eq 'no') {
+        if($nodataconn425) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "425 Can't open data connection\r\n";
+        }
+        elsif($nodataconn421) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "421 Connection timed out\r\n";
+        }
+        elsif($nodataconn150) {
+            sendcontrol "150 Opening data connection\r\n";
+            # client shall timeout
+        }
+        else {
+            # client shall timeout
+        }
+        return 0;
+    }
+
     if($ftplistparserstate) {
         my @content = wildcard_getfile($ftptargetdir, $testno);
         if($content[0] == -1) {
@@ -1002,6 +1288,25 @@ sub STOR_ftp {
 
     my $filename = "log/upload.$testno";
 
+    if($datasockf_conn eq 'no') {
+        if($nodataconn425) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "425 Can't open data connection\r\n";
+        }
+        elsif($nodataconn421) {
+            sendcontrol "150 Opening data connection\r\n";
+            sendcontrol "421 Connection timed out\r\n";
+        }
+        elsif($nodataconn150) {
+            sendcontrol "150 Opening data connection\r\n";
+            # client shall timeout
+        }
+        else {
+            # client shall timeout
+        }
+        return 0;
+    }
+
     logmsg "STOR test number $testno in $filename\n";
 
     sendcontrol "125 Gimme gimme gimme!\r\n";
@@ -1022,7 +1327,7 @@ sub STOR_ftp {
                 $size = hex($1);
             }
 
-            sysread DREAD, $line, $size;
+            read_datasockf(\$line, $size);
 
             #print STDERR "  GOT: $size bytes\n";
 
@@ -1053,31 +1358,61 @@ sub STOR_ftp {
 sub PASV_ftp {
     my ($arg, $cmd)=@_;
     my $pasvport;
+    my $bindonly = ($nodataconn) ? '--bindonly' : '';
 
     # kill previous data connection sockfilt when alive
-    killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+    if($datasockf_runs eq 'yes') {
+        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt for $datasockf_mode data channel killed\n";
+    }
+    datasockf_state('STOPPED');
+
+    logmsg "====> Passive DATA channel requested by client\n";
+
+    logmsg "DATA sockfilt for passive data channel starting...\n";
 
     # We fire up a new sockfilt to do the data transfer for us.
     my $datasockfcmd = "./server/sockfilt " .
-        "--ipv$ipvnum --port 0 " .
+        "--ipv$ipvnum $bindonly --port 0 " .
         "--pidfile \"$datasockf_pidfile\" " .
         "--logfile \"$datasockf_logfile\"";
     $slavepid = open2(\*DREAD, \*DWRITE, $datasockfcmd);
 
+    if($nodataconn) {
+        datasockf_state('PASSIVE_NODATACONN');
+    }
+    else {
+        datasockf_state('PASSIVE');
+    }
+
+    print STDERR "$datasockfcmd\n" if($verbose);
+
     print DWRITE "PING\n";
     my $pong;
     sysread_or_die(\*DREAD, \$pong, 5);
 
-    if($pong !~ /^PONG/) {
-        logmsg "failed to run sockfilt for data connection\n";
-        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+    if($pong =~ /^FAIL/) {
+        logmsg "DATA sockfilt said: FAIL\n";
+        logmsg "DATA sockfilt for passive data channel failed\n";
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
         sendcontrol "500 no free ports!\r\n";
-        return 0;
+        return;
+    }
+    elsif($pong !~ /^PONG/) {
+        logmsg "DATA sockfilt unexpected response: $pong\n";
+        logmsg "DATA sockfilt for passive data channel failed\n";
+        logmsg "DATA sockfilt killed now\n";
+        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
+        sendcontrol "500 no free ports!\r\n";
+        return;
     }
 
-    logmsg "Run sockfilt for data on pid $slavepid\n";
+    logmsg "DATA sockfilt for passive data channel started (pid $slavepid)\n";
 
-    # Find out what port we listen on
+    # Find out on what port we listen on or have bound
     my $i;
     print DWRITE "PORT\n";
 
@@ -1093,7 +1428,7 @@ sub PASV_ftp {
     }
 
     # READ the response data
-    sysread_or_die(\*DREAD, \$i, $size);
+    read_datasockf(\$i, $size);
 
     # The data is in the format
     # IPvX/NNN
@@ -1103,6 +1438,27 @@ sub PASV_ftp {
         $pasvport = $2;
     }
 
+    if(!$pasvport) {
+        logmsg "DATA sockfilt unknown listener port\n";
+        logmsg "DATA sockfilt for passive data channel failed\n";
+        logmsg "DATA sockfilt killed now\n";
+        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
+        sendcontrol "500 no free ports!\r\n";
+        return;
+    }
+
+    if($nodataconn) {
+        my $str = nodataconn_str();
+        logmsg "DATA sockfilt for passive data channel ($str) bound on port ".
+               "$pasvport\n";
+    }
+    else {
+        logmsg "DATA sockfilt for passive data channel listens on port ".
+               "$pasvport\n";
+    }
+
     if($cmd ne "EPSV") {
         # PASV reply
         my $p=$listenaddr;
@@ -1111,13 +1467,22 @@ sub PASV_ftp {
             $p="1,2,3,4";
         }
         sendcontrol sprintf("227 Entering Passive Mode ($p,%d,%d)\n",
-                            ($pasvport/256), ($pasvport%256));
+                            int($pasvport/256), int($pasvport%256));
     }
     else {
         # EPSV reply
         sendcontrol sprintf("229 Entering Passive Mode (|||%d|)\n", $pasvport);
     }
 
+    logmsg "Client has been notified that DATA conn ".
+           "will be accepted on port $pasvport\n";
+
+    if($nodataconn) {
+        my $str = nodataconn_str();
+        logmsg "====> Client fooled ($str)\n";
+        return;
+    }
+
     eval {
         local $SIG{ALRM} = sub { die "alarm\n" };
 
@@ -1127,6 +1492,8 @@ sub PASV_ftp {
         # Wait for 'CNCT'
         my $input;
 
+        # FIX: Monitor ctrl conn for disconnect
+
         while(sysread(DREAD, $input, 5)) {
 
             if($input !~ /^CNCT/) {
@@ -1134,7 +1501,7 @@ sub PASV_ftp {
                 logmsg "Odd, we got $input from client\n";
                 next;
             }
-            logmsg "====> Client DATA connect\n";
+            logmsg "Client connects to port $pasvport\n";
             last;
         }
         alarm 0;
@@ -1144,54 +1511,85 @@ sub PASV_ftp {
         logmsg "$srvrname server timed out awaiting data connection ".
             "on port $pasvport\n";
         logmsg "accept failed or connection not even attempted\n";
+        logmsg "DATA sockfilt killed now\n";
         killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
         return;
     }
     else {
-        logmsg "data connection setup on port $pasvport\n";
+        logmsg "====> Client established passive DATA connection ".
+               "on port $pasvport\n";
     }
 
     return;
 }
 
-# Support both PORT and EPRT here. Consider LPRT too.
+#
+# Support both PORT and EPRT here.
+#
 
 sub PORT_ftp {
     my ($arg, $cmd) = @_;
     my $port;
     my $addr;
 
+    # kill previous data connection sockfilt when alive
+    if($datasockf_runs eq 'yes') {
+        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt for $datasockf_mode data channel killed\n";
+    }
+    datasockf_state('STOPPED');
+
+    logmsg "====> Active DATA channel requested by client\n";
+
     # We always ignore the given IP and use localhost.
 
     if($cmd eq "PORT") {
         if($arg !~ /(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)/) {
-            logmsg "bad PORT-line: $arg\n";
+            logmsg "DATA sockfilt for active data channel not started ".
+                   "(bad PORT-line: $arg)\n";
             sendcontrol "500 silly you, go away\r\n";
-            return 0;
+            return;
         }
         $port = ($5<<8)+$6;
         $addr = "$1.$2.$3.$4";
     }
     # EPRT |2|::1|49706|
-    elsif(($cmd eq "EPRT") && ($grok_eprt)) {
+    elsif($cmd eq "EPRT") {
         if($arg !~ /(\d+)\|([^\|]+)\|(\d+)/) {
+            logmsg "DATA sockfilt for active data channel not started ".
+                   "(bad EPRT-line: $arg)\n";
             sendcontrol "500 silly you, go away\r\n";
-            return 0;
+            return;
         }
         sendcontrol "200 Thanks for dropping by. We contact you later\r\n";
         $port = $3;
         $addr = $2;
     }
     else {
+        logmsg "DATA sockfilt for active data channel not started ".
+               "(invalid command: $cmd)\n";
         sendcontrol "500 we don't like $cmd now\r\n";
-        return 0;
+        return;
     }
 
     if(!$port || $port > 65535) {
-        print STDERR "very illegal PORT number: $port\n";
-        return 1;
+        logmsg "DATA sockfilt for active data channel not started ".
+               "(illegal PORT number: $port)\n";
+        return;
     }
 
+    if($nodataconn) {
+        my $str = nodataconn_str();
+        logmsg "DATA sockfilt for active data channel not started ($str)\n";
+        datasockf_state('ACTIVE_NODATACONN');
+        logmsg "====> Active DATA channel not established\n";
+        return;
+    }
+
+    logmsg "DATA sockfilt for active data channel starting...\n";
+
     # We fire up a new sockfilt to do the data transfer for us.
     my $datasockfcmd = "./server/sockfilt " .
         "--ipv$ipvnum --connect $port --addr \"$addr\" " .
@@ -1199,22 +1597,107 @@ sub PORT_ftp {
         "--logfile \"$datasockf_logfile\"";
     $slavepid = open2(\*DREAD, \*DWRITE, $datasockfcmd);
 
+    datasockf_state('ACTIVE');
+
     print STDERR "$datasockfcmd\n" if($verbose);
 
     print DWRITE "PING\n";
     my $pong;
     sysread_or_die(\*DREAD, \$pong, 5);
 
-    if($pong !~ /^PONG/) {
-        logmsg "Failed sockfilt for data connection\n";
+    if($pong =~ /^FAIL/) {
+        logmsg "DATA sockfilt said: FAIL\n";
+        logmsg "DATA sockfilt for active data channel failed\n";
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
+        # client shall timeout awaiting connection from server
+        return;
+    }
+    elsif($pong !~ /^PONG/) {
+        logmsg "DATA sockfilt unexpected response: $pong\n";
+        logmsg "DATA sockfilt for active data channel failed\n";
+        logmsg "DATA sockfilt killed now\n";
         killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt not running\n";
+        datasockf_state('STOPPED');
+        # client shall timeout awaiting connection from server
+        return;
     }
 
-    logmsg "====> Client DATA connect to port $port\n";
+    logmsg "DATA sockfilt for active data channel started (pid $slavepid)\n";
+
+    logmsg "====> Active DATA channel connected to client port $port\n";
 
     return;
 }
 
+#**********************************************************************
+# datasockf_state is used to change variables that keep state info
+# relative to the FTP secondary or data sockfilt process as soon as
+# one of the five possible stable states is reached. Variables that
+# are modified by this sub may be checked independently but should
+# not be changed except by calling this sub.
+#
+sub datasockf_state {
+    my $state = $_[0];
+
+  if($state eq 'STOPPED') {
+    # Data sockfilter initial state, not running,
+    # not connected and not used.
+    $datasockf_state = $state;
+    $datasockf_mode = 'none';
+    $datasockf_runs = 'no';
+    $datasockf_conn = 'no';
+  }
+  elsif($state eq 'PASSIVE') {
+    # Data sockfilter accepted connection from client.
+    $datasockf_state = $state;
+    $datasockf_mode = 'passive';
+    $datasockf_runs = 'yes';
+    $datasockf_conn = 'yes';
+  }
+  elsif($state eq 'ACTIVE') {
+    # Data sockfilter has connected to client.
+    $datasockf_state = $state;
+    $datasockf_mode = 'active';
+    $datasockf_runs = 'yes';
+    $datasockf_conn = 'yes';
+  }
+  elsif($state eq 'PASSIVE_NODATACONN') {
+    # Data sockfilter bound port without listening,
+    # client won't be able to establish data connection.
+    $datasockf_state = $state;
+    $datasockf_mode = 'passive';
+    $datasockf_runs = 'yes';
+    $datasockf_conn = 'no';
+  }
+  elsif($state eq 'ACTIVE_NODATACONN') {
+    # Data sockfilter does not even run,
+    # client awaits data connection from server in vain.
+    $datasockf_state = $state;
+    $datasockf_mode = 'active';
+    $datasockf_runs = 'no';
+    $datasockf_conn = 'no';
+  }
+  else {
+      die "Internal error. Unknown datasockf state: $state!";
+  }
+}
+
+#**********************************************************************
+# nodataconn_str returns string of efective nodataconn command. Notice
+# that $nodataconn may be set alone or in addition to a $nodataconnXXX.
+#
+sub nodataconn_str {
+    my $str;
+    # order matters
+    $str = 'NODATACONN' if($nodataconn);
+    $str = 'NODATACONN425' if($nodataconn425);
+    $str = 'NODATACONN421' if($nodataconn421);
+    $str = 'NODATACONN150' if($nodataconn150);
+    return "$str";
+}
+
 #**********************************************************************
 # customize configures test server operation for each curl test, reading
 # configuration commands/parameters from server commands file each time
@@ -1222,15 +1705,21 @@ sub PORT_ftp {
 # On success returns 1, otherwise zero.
 #
 sub customize {
-    $ctrldelay = 0;    # default is no throttling of the ctrl stream
-    $datadelay = 0;    # default is no throttling of the data stream
-    $retrweirdo = 0;   # default is no use of RETRWEIRDO
-    $retrnosize = 0;   # default is no use of RETRNOSIZE
-    $pasvbadip = 0;    # default is no use of PASVBADIP
-    $nosave = 0;       # default is to actually save uploaded data to file
-    %customreply = (); #
-    %customcount = (); #
-    %delayreply = ();  #
+    $ctrldelay = 0;     # default is no throttling of the ctrl stream
+    $datadelay = 0;     # default is no throttling of the data stream
+    $retrweirdo = 0;    # default is no use of RETRWEIRDO
+    $retrnosize = 0;    # default is no use of RETRNOSIZE
+    $pasvbadip = 0;     # default is no use of PASVBADIP
+    $nosave = 0;        # default is to actually save uploaded data to file
+    $nodataconn = 0;    # default is to establish or accept data channel
+    $nodataconn425 = 0; # default is to not send 425 without data channel
+    $nodataconn421 = 0; # default is to not send 421 without data channel
+    $nodataconn150 = 0; # default is to not send 150 without data channel
+    $support_capa = 0;  # default is to not support capability command
+    $support_auth = 0;  # default is to not support authentication command
+    %customreply = ();  #
+    %customcount = ();  #
+    %delayreply = ();   #
 
     open(CUSTOM, "<log/ftpserver.cmd") ||
         return 1;
@@ -1242,7 +1731,7 @@ sub customize {
             $customreply{$1}=eval "qq{$2}";
             logmsg "FTPD: set custom reply for $1\n";
         }
-        if($_ =~ /COUNT ([A-Z]+) (.*)/) {
+        elsif($_ =~ /COUNT ([A-Z]+) (.*)/) {
             # we blank the customreply for this command when having
             # been used this number of times
             $customcount{$1}=$2;
@@ -1269,6 +1758,37 @@ sub customize {
             logmsg "FTPD: instructed to use PASVBADIP\n";
             $pasvbadip=1;
         }
+        elsif($_ =~ /NODATACONN425/) {
+            # applies to both active and passive FTP modes
+            logmsg "FTPD: instructed to use NODATACONN425\n";
+            $nodataconn425=1;
+            $nodataconn=1;
+        }
+        elsif($_ =~ /NODATACONN421/) {
+            # applies to both active and passive FTP modes
+            logmsg "FTPD: instructed to use NODATACONN421\n";
+            $nodataconn421=1;
+            $nodataconn=1;
+        }
+        elsif($_ =~ /NODATACONN150/) {
+            # applies to both active and passive FTP modes
+            logmsg "FTPD: instructed to use NODATACONN150\n";
+            $nodataconn150=1;
+            $nodataconn=1;
+        }
+        elsif($_ =~ /NODATACONN/) {
+            # applies to both active and passive FTP modes
+            logmsg "FTPD: instructed to use NODATACONN\n";
+            $nodataconn=1;
+        }
+        elsif($_ =~ /SUPPORTCAPA/) {
+            logmsg "FTPD: instructed to support CAPABILITY command\n";
+            $support_capa=1;
+        }
+        elsif($_ =~ /SUPPORTAUTH/) {
+            logmsg "FTPD: instructed to support AUTHENTICATION command\n";
+            $support_auth=1;
+        }
         elsif($_ =~ /NOSAVE/) {
             # don't actually store the file we upload - to be used when
             # uploading insanely huge amounts
@@ -1341,12 +1861,10 @@ while(@ARGV) {
     elsif($ARGV[0] eq '--ipv4') {
         $ipvnum = 4;
         $listenaddr = '127.0.0.1' if($listenaddr eq '::1');
-        $grok_eprt = 0;
     }
     elsif($ARGV[0] eq '--ipv6') {
         $ipvnum = 6;
         $listenaddr = '::1' if($listenaddr eq '127.0.0.1');
-        $grok_eprt = 1;
     }
     elsif($ARGV[0] eq '--port') {
         if($ARGV[1] && ($ARGV[1] =~ /^(\d+)$/)) {
@@ -1420,6 +1938,14 @@ logmsg("logged pid $$ in $pidfile\n");
 
 
 while(1) {
+
+    # kill previous data connection sockfilt when alive
+    if($datasockf_runs eq 'yes') {
+        killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
+        logmsg "DATA sockfilt for $datasockf_mode data channel killed now\n";
+    }
+    datasockf_state('STOPPED');
+
     #
     # We read 'sockfilt' commands.
     #
@@ -1430,7 +1956,7 @@ while(1) {
 
     if($input !~ /^CNCT/) {
         # we wait for a connected client
-        logmsg "sockfilt said: $input";
+        logmsg "MAIN sockfilt said: $input";
         next;
     }
     logmsg "====> Client connect\n";
@@ -1441,8 +1967,6 @@ while(1) {
     # flush data:
     $| = 1;
 
-    killsockfilters($proto, $ipvnum, $idnum, $verbose, 'data');
-
     &customize(); # read test control instructions
 
     sendcontrol @welcome;
@@ -1461,6 +1985,8 @@ while(1) {
         }
     }
 
+    my $full = "";
+
     while(1) {
         my $i;
 
@@ -1471,7 +1997,7 @@ while(1) {
         sysread_or_die(\*SFREAD, \$i, 5);
 
         if($i !~ /^DATA/) {
-            logmsg "sockfilt said $i";
+            logmsg "MAIN sockfilt said $i";
             if($i =~ /^DISC/) {
                 # disconnect
                 last;
@@ -1488,38 +2014,42 @@ while(1) {
         }
 
         # data
-        sysread SFREAD, $_, $size;
+        read_mainsockf(\$input, $size);
 
-        ftpmsg $_;
+        ftpmsg $input;
+
+        $full .= $input;
+
+        # Loop until command completion
+        next unless($full =~ /\r\n$/);
 
         # Remove trailing CRLF.
-        s/[\n\r]+$//;
+        $full =~ s/[\n\r]+$//;
 
         my $FTPCMD;
         my $FTPARG;
-        my $full=$_;
         if($proto eq "imap") {
             # IMAP is different with its identifier first on the command line
-            unless (m/^([^ ]+) ([^ ]+) (.*)/ ||
-                    m/^([^ ]+) ([^ ]+)/) {
-                sendcontrol "$1 '$_': command not understood.\r\n";
+            unless(($full =~ /^([^ ]+) ([^ ]+) (.*)/) ||
+                   ($full =~ /^([^ ]+) ([^ ]+)/)) {
+                sendcontrol "$1 '$full': command not understood.\r\n";
                 last;
             }
             $cmdid=$1; # set the global variable
             $FTPCMD=$2;
             $FTPARG=$3;
         }
-        elsif (m/^([A-Z]{3,4})(\s(.*))?$/i) {
+        elsif($full =~ /^([A-Z]{3,4})(\s(.*))?$/i) {
             $FTPCMD=$1;
             $FTPARG=$3;
         }
-        elsif($proto eq "smtp" && m/^[A-Z0-9+\/]{0,512}={0,2}$/i) {
+        elsif(($proto eq "smtp") && ($full =~ /^[A-Z0-9+\/]{0,512}={0,2}$/i)) {
             # SMTP long "commands" are base64 authentication data.
-            $FTPCMD=$_;
+            $FTPCMD=$full;
             $FTPARG="";
         }
         else {
-            sendcontrol "500 '$_': command not understood.\r\n";
+            sendcontrol "500 '$full': command not understood.\r\n";
             last;
         }
 
@@ -1529,6 +2059,8 @@ while(1) {
             print STDERR "IN: $full\n";
         }
 
+        $full = "";
+
         my $delay = $delayreply{$FTPCMD};
         if($delay) {
             # just go sleep this many seconds!
@@ -1576,7 +2108,15 @@ while(1) {
 
         if($check) {
             logmsg "$FTPCMD wasn't handled!\n";
-            sendcontrol "500 $FTPCMD is not dealt with!\r\n";
+            if($proto eq 'pop3') {
+                sendcontrol "-ERR $FTPCMD is not dealt with!\r\n";
+            }
+            elsif($proto eq 'imap') {
+                sendcontrol "$cmdid BAD $FTPCMD is not dealt with!\r\n";
+            }
+            else {
+                sendcontrol "500 $FTPCMD is not dealt with!\r\n";
+            }
         }
 
     } # while(1)
